/*
 * Copyright (c) CERN 2013-2017
 *
 * Copyright (c) Members of the EMI Collaboration. 2011-2013
 *  See  http://www.eu-emi.eu/partners for details on the copyright
 *  holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <apr_strings.h>
#include <ctype.h>
#include <dmlite/c/catalog.h>
#include <dmlite/c/checksums.h>
#include <httpd.h>
#include <regex.h>
#include <shared/utils.h>
#include <http_log.h>

#define RFC3230_REGEXP "^([[:alnum:]]+)(;q=[[:digit:]]*)?([,]?)"

static char* dav_shared_strcpy_lower(char *out, const char *in, size_t max)
{
    size_t i;
    for (i = 0; in[i] && i < max; ++i)
        out[i] = tolower(in[i]);
    return out;
}


int dav_shared_next_digest(const char **want_digest, char *output, size_t outsize)
{
    regex_t regex;
    regmatch_t matches[3];

    if (regcomp(&regex, RFC3230_REGEXP, REG_EXTENDED) != 0)
        abort();

    while (isspace(**want_digest))
        ++(*want_digest);

    if (regexec(&regex, *want_digest, 3, matches, 0) == 0) {
        size_t digest_len = matches[1].rm_eo - matches[1].rm_so;
        const char *digest = (*want_digest) + matches[1].rm_so;

        if (outsize < digest_len) {
            digest_len = outsize;
        }

        dav_shared_strcpy_lower(output, digest, digest_len);
        output[digest_len] = '\0';

        *want_digest += matches[0].rm_eo;

        return 1;
    }

    return 0;
}
